=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2011-06-28 13:13:05 +0000
+++ loggerhead/apps/branch.py	2011-06-28 16:09:37 +0000
@@ -164,6 +164,9 @@
                 self.absolute_url('/changes'))
         if path == 'static':
             return static_app
+        elif path == '+json':
+            environ['loggerhead.as_json'] = True
+            path = request.path_info_pop(environ)
         cls = self.controllers_dict.get(path)
         if cls is None:
             raise httpexceptions.HTTPNotFound()

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2011-06-28 13:13:05 +0000
+++ loggerhead/controllers/__init__.py	2011-06-28 16:09:37 +0000
@@ -18,6 +18,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import bzrlib.errors
+import simplejson
 import time
 
 from paste.httpexceptions import HTTPNotFound
@@ -53,6 +54,7 @@
 class TemplatedBranchView(object):
 
     template_path = None
+    supports_json = False
 
     def __init__(self, branch, history_callable):
         self._branch = branch
@@ -95,13 +97,17 @@
 
     def __call__(self, environ, start_response):
         z = time.time()
+        if environ.get('loggerhead.as_json') and not self.supports_json:
+            raise HTTPNotFound
         path = self.parse_args(environ)
         headers = {}
         values = self.get_values(path, self.kwargs, headers)
 
         self.log.info('Getting information for %s: %.3f secs' % (
             self.__class__.__name__, time.time() - z))
-        if 'Content-Type' not in headers:
+        if environ.get('loggerhead.as_json'):
+            headers['Content-Type'] = 'application/json'
+        elif 'Content-Type' not in headers:
             headers['Content-Type'] = 'text/html'
         writer = start_response("200 OK", headers.items())
         if environ.get('REQUEST_METHOD') == 'HEAD':
@@ -109,9 +115,13 @@
             return []
         z = time.time()
         w = BufferingWriter(writer, 8192)
-        self.add_template_values(values)
-        template = load_template(self.template_path)
-        template.expand_into(w, **values)
+        if environ.get('loggerhead.as_json'):
+            w.write(simplejson.dumps(values,
+                default=util.convert_to_json_ready))
+        else:
+            self.add_template_values(values)
+            template = load_template(self.template_path)
+            template.expand_into(w, **values)
         w.flush()
         self.log.info(
             'Rendering %s: %.3f secs, %s bytes' % (

=== modified file 'loggerhead/controllers/filediff_ui.py'
--- loggerhead/controllers/filediff_ui.py	2009-07-07 23:52:24 +0000
+++ loggerhead/controllers/filediff_ui.py	2011-06-27 15:05:06 +0000
@@ -77,6 +77,7 @@
 class FileDiffUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.filediff'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         revid = urllib.unquote(self.args[0])
@@ -87,6 +88,5 @@
             self._history._branch.repository, file_id, compare_revid, revid)
 
         return {
-            'util': util,
             'chunks': chunks,
         }

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2010-07-12 15:12:04 +0000
+++ loggerhead/controllers/inventory_ui.py	2011-06-27 16:30:57 +0000
@@ -42,6 +42,7 @@
 class InventoryUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.inventory'
+    supports_json = True
 
     def get_filelist(self, inv, path, sort_type, revno_url):
         """
@@ -78,6 +79,11 @@
                 absolutepath = path + '/' + pathname
             revid = entry.revision
 
+            # TODO: For the JSON rendering, this inlines the "change" aka
+            # revision information attached to each file. Consider either
+            # pulling this out as a separate changes dict, or possibly just
+            # including the revision id and having a separate request to get
+            # back the revision info.
             file = util.Container(
                 filename=filename, executable=entry.executable,
                 kind=entry.kind, absolutepath=absolutepath,
@@ -110,9 +116,6 @@
         start_revid = kwargs.get('start_revid', None)
         sort_type = kwargs.get('sort', 'filename')
 
-        # no navbar for revisions
-        navigation = util.Container()
-
         if path is not None:
             path = path.rstrip('/')
             file_id = rev_tree.path2id(path)
@@ -133,14 +136,7 @@
         else:
             updir = dirname(path)
 
-        # Directory Breadcrumbs
-        directory_breadcrumbs = util.directory_breadcrumbs(
-                self._branch.friendly_name,
-                self._branch.is_root,
-                'files')
-
         if not is_null_rev(revid):
-
             change = history.get_changes([ revid ])[0]
             # If we're looking at the tip, use head: in the URL instead
             if revid == branch.last_revision():
@@ -148,32 +144,49 @@
             else:
                 revno_url = history.get_revno(revid)
             history.add_branch_nicks(change)
-
-            # Create breadcrumb trail for the path within the branch
-            branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
             filelist = self.get_filelist(rev_tree.inventory, path, sort_type, revno_url)
+
         else:
             start_revid = None
             change = None
             path = "/"
             updir = None
             revno_url = 'head:'
-            branch_breadcrumbs = []
             filelist = []
 
         return {
-            'branch': self._branch,
-            'util': util,
             'revid': revid,
             'revno_url': revno_url,
             'change': change,
             'path': path,
             'updir': updir,
             'filelist': filelist,
-            'navigation': navigation,
-            'url': self._branch.context_url,
             'start_revid': start_revid,
+        }
+
+    def add_template_values(self, values):
+        super(InventoryUI, self).add_template_values(values)
+        # Directory Breadcrumbs
+        directory_breadcrumbs = util.directory_breadcrumbs(
+                self._branch.friendly_name,
+                self._branch.is_root,
+                'files')
+
+        path = values['path']
+        revid = values['revid']
+        # no navbar for revisions
+        navigation = util.Container()
+
+        if is_null_rev(revid):
+            branch_breadcrumbs = []
+        else:
+            # Create breadcrumb trail for the path within the branch
+            branch = self._history._branch
+            rev_tree = branch.repository.revision_tree(revid)
+            branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
+        values.update({
             'fileview_active': True,
             'directory_breadcrumbs': directory_breadcrumbs,
             'branch_breadcrumbs': branch_breadcrumbs,
-        }
+            'navigation': navigation,
+        })

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2011-03-02 14:07:21 +0000
+++ loggerhead/controllers/revision_ui.py	2011-06-27 17:11:24 +0000
@@ -36,6 +36,7 @@
 class RevisionUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revision'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         h = self._history
@@ -44,9 +45,10 @@
         filter_file_id = kwargs.get('filter_file_id', None)
         start_revid = h.fix_revid(kwargs.get('start_revid', None))
         query = kwargs.get('q', None)
-        remember = h.fix_revid(kwargs.get('remember', None))
         compare_revid = h.fix_revid(kwargs.get('compare_revid', None))
 
+        # TODO: This try/except looks to date before real exception handling
+        # and should be removed
         try:
             revid, start_revid, revid_list = h.get_view(revid,
                                                         start_revid,
@@ -55,26 +57,64 @@
         except:
             self.log.exception('Exception fetching changes')
             raise HTTPServerError('Could not fetch changes')
-
+        # XXX: Some concern about namespace collisions. These are only stored
+        # here so they can be expanded into the template later. Should probably
+        # be stored in a specific dict/etc.
+        self.revid_list = revid_list
+        self.compare_revid = compare_revid
+        self.path = path
+        kwargs['start_revid'] = start_revid
+
+        change = h.get_changes([revid])[0]
+
+        if compare_revid is None:
+            file_changes = h.get_file_changes(change)
+        else:
+            file_changes = h.file_changes_for_revision_ids(
+                compare_revid, change.revid)
+
+        h.add_branch_nicks(change)
+
+        if '.' in change.revno:
+            # Walk "up" though the merge-sorted graph until we find a
+            # revision with merge depth 0: this is the revision that merged
+            # this one to mainline.
+            ri = self._history._rev_info
+            i = self._history._rev_indices[change.revid]
+            while ri[i][0][2] > 0:
+                i -= 1
+            merged_in = ri[i][0][3]
+        else:
+            merged_in = None
+
+        return {
+            'revid': revid,
+            'change': change,
+            'file_changes': file_changes,
+            'merged_in': merged_in,
+        }
+
+    def add_template_values(self, values):
+        super(RevisionUI, self).add_template_values(values)
+        remember = self._history.fix_revid(self.kwargs.get('remember', None))
+        query = self.kwargs.get('q', None)
+        filter_file_id = self.kwargs.get('filter_file_id', None)
+        start_revid = self.kwargs['start_revid']
         navigation = util.Container(
-            revid_list=revid_list, revid=revid, start_revid=start_revid,
+            revid_list=self.revid_list, revid=values['revid'],
+            start_revid=start_revid,
             filter_file_id=filter_file_id, pagesize=1,
-            scan_url='/revision', branch=self._branch, feed=True, history=h)
+            scan_url='/revision', branch=self._branch, feed=True,
+            history=self._history)
         if query is not None:
             navigation.query = query
         util.fill_in_navigation(navigation)
-
-        change = h.get_changes([revid])[0]
-
-        if compare_revid is None:
-            file_changes = h.get_file_changes(change)
-        else:
-            file_changes = h.file_changes_for_revision_ids(
-                compare_revid, change.revid)
-
+        path = self.path
         if path in ('', '/'):
             path = None
 
+
+        file_changes = values['file_changes']
         link_data = {}
         path_to_id = {}
         if path:
@@ -90,20 +130,6 @@
                     dq(item.new_revision), dq(item.old_revision), dq(item.file_id))
                 path_to_id[item.filename] = 'diff-' + str(i)
 
-        h.add_branch_nicks(change)
-
-        if '.' in change.revno:
-            # Walk "up" though the merge-sorted graph until we find a
-            # revision with merge depth 0: this is the revision that merged
-            # this one to mainline.
-            ri = self._history._rev_info
-            i = self._history._rev_indices[change.revid]
-            while ri[i][0][2] > 0:
-                i -= 1
-            merged_in = ri[i][0][3]
-        else:
-            merged_in = None
-
         # Directory Breadcrumbs
         directory_breadcrumbs = (
             util.directory_breadcrumbs(
@@ -111,25 +137,18 @@
                 self._branch.is_root,
                 'changes'))
 
-        return {
-            'branch': self._branch,
-            'revid': revid,
-            'change': change,
-            'file_changes': file_changes,
-            'diff_chunks': diff_chunks,
+        values.update({
+            'history': self._history,
             'link_data': simplejson.dumps(link_data),
-            'specific_path': path,
             'json_specific_path': simplejson.dumps(path),
             'path_to_id': simplejson.dumps(path_to_id),
-            'start_revid': start_revid,
+            'directory_breadcrumbs': directory_breadcrumbs,
+            'navigation': navigation,
+            'remember': remember,
+            'compare_revid': self.compare_revid,
             'filter_file_id': filter_file_id,
-            'util': util,
-            'history': h,
-            'merged_in': merged_in,
-            'navigation': navigation,
+            'diff_chunks': diff_chunks,
             'query': query,
-            'remember': remember,
-            'compare_revid': compare_revid,
-            'url': self._branch.context_url,
-            'directory_breadcrumbs': directory_breadcrumbs,
-        }
+            'specific_path': path,
+            'start_revid': start_revid,
+        })

=== modified file 'loggerhead/controllers/revlog_ui.py'
--- loggerhead/controllers/revlog_ui.py	2009-03-19 00:44:05 +0000
+++ loggerhead/controllers/revlog_ui.py	2011-06-27 15:05:06 +0000
@@ -1,12 +1,12 @@
 import urllib
 
-from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
 
 
 class RevLogUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revlog'
+    supports_json = True
 
     def get_values(self, path, kwargs, headers):
         history = self._history
@@ -18,10 +18,7 @@
         history.add_branch_nicks(change)
 
         return {
-            'branch': self._branch,
             'entry': change,
             'file_changes': file_changes,
-            'util': util,
             'revid': revid,
-            'url': self._branch.context_url,
         }

=== modified file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	2011-06-28 10:58:27 +0000
+++ loggerhead/tests/test_controllers.py	2011-06-28 16:06:12 +0000
@@ -1,47 +1,27 @@
-from cStringIO import StringIO
-import logging
-
-from paste.httpexceptions import HTTPServerError
-
-from bzrlib import errors
+import simplejson
 
 from loggerhead.apps.branch import BranchWSGIApp
 from loggerhead.controllers.annotate_ui import AnnotateUI
 from loggerhead.controllers.inventory_ui import InventoryUI
 from loggerhead.controllers.revision_ui import RevisionUI
-from loggerhead.tests.test_simple import BasicTests
+from loggerhead.tests.test_simple import BasicTests, consume_app
 from loggerhead import util
 
 
 class TestInventoryUI(BasicTests):
 
+    def make_bzrbranch_for_tree_shape(self, shape):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(shape)
+        tree.smart_add([])
+        tree.commit('')
+        self.addCleanup(tree.branch.lock_read().unlock)
+        return tree.branch
+
     def make_bzrbranch_and_inventory_ui_for_tree_shape(self, shape):
-        tree = self.make_branch_and_tree('.')
-        self.build_tree(shape)
-        tree.smart_add([])
-        tree.commit('')
-        tree.branch.lock_read()
-        self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch, '')
-        branch_app.log.setLevel(logging.CRITICAL)
-        # These are usually set in BranchWSGIApp.app(), which is set from env
-        # settings set by BranchesFromTransportRoot, so we fake it.
-        branch_app._static_url_base = '/'
-        branch_app._url_base = '/'
-        return tree.branch, InventoryUI(branch_app, branch_app.get_history)
-
-    def consume_app(self, app, extra_environ=None):
-        env = {'SCRIPT_NAME': '/files', 'PATH_INFO': ''}
-        if extra_environ is not None:
-            env.update(extra_environ)
-        body = StringIO()
-        start = []
-        def start_response(status, headers, exc_info=None):
-            start.append((status, headers, exc_info))
-            return body.write
-        extra_content = list(app(env, start_response))
-        body.writelines(extra_content)
-        return start[0], body.getvalue()
+        branch = self.make_bzrbranch_for_tree_shape(shape)
+        branch_app = self.make_branch_app(branch)
+        return branch, InventoryUI(branch_app, branch_app.get_history)
 
     def test_get_filelist(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
@@ -52,7 +32,8 @@
     def test_smoke(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
-        start, content = self.consume_app(inv_ui)
+        start, content = consume_app(inv_ui,
+            {'SCRIPT_NAME': '/files', 'PATH_INFO': ''})
         self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
                          start)
         self.assertContainsRe(content, 'filename')
@@ -60,43 +41,74 @@
     def test_no_content_for_HEAD(self):
         bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
             ['filename'])
-        start, content = self.consume_app(inv_ui,
-                            extra_environ={'REQUEST_METHOD': 'HEAD'})
+        start, content = consume_app(inv_ui,
+            {'SCRIPT_NAME': '/files', 'PATH_INFO': '',
+             'REQUEST_METHOD': 'HEAD'})
         self.assertEqual(('200 OK', [('Content-Type', 'text/html')], None),
                          start)
         self.assertEqual('', content)
 
+    def test_get_values_smoke(self):
+        branch = self.make_bzrbranch_for_tree_shape(['a-file'])
+        branch_app = self.make_branch_app(branch)
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/files'}
+        inv_ui = branch_app.lookup_app(env)
+        inv_ui.parse_args(env)
+        values = inv_ui.get_values('', {}, {})
+        self.assertEqual('a-file', values['filelist'][0].filename)
+
+    def test_json_render_smoke(self):
+        branch = self.make_bzrbranch_for_tree_shape(['a-file'])
+        branch_app = self.make_branch_app(branch)
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/files'}
+        inv_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(inv_ui, env)
+
 
 class TestRevisionUI(BasicTests):
 
-    def make_bzrbranch_and_revision_ui_for_tree_shapes(self, shape1, shape2):
+    def make_branch_app_for_revision_ui(self, shape1, shape2):
         tree = self.make_branch_and_tree('.')
         self.build_tree_contents(shape1)
         tree.smart_add([])
-        tree.commit('')
+        tree.commit('msg 1', rev_id='rev-1')
         self.build_tree_contents(shape2)
         tree.smart_add([])
-        tree.commit('')
-        tree.branch.lock_read()
-        self.addCleanup(tree.branch.unlock)
-        branch_app = BranchWSGIApp(tree.branch)
-        branch_app._environ = {
-            'wsgi.url_scheme':'',
-            'SERVER_NAME':'',
-            'SERVER_PORT':'80',
-            }
-        branch_app._url_base = ''
-        branch_app.friendly_name = ''
-        return tree.branch, RevisionUI(branch_app, branch_app.get_history)
+        tree.commit('msg 2', rev_id='rev-2')
+        branch = tree.branch
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
 
     def test_get_values(self):
-        branch, rev_ui = self.make_bzrbranch_and_revision_ui_for_tree_shapes(
-            [], [])
-        rev_ui.args = ['2']
-        util.set_context({})
-        self.assertIsInstance(
-            rev_ui.get_values('', {}, []),
-            dict)
+        branch_app = self.make_branch_app_for_revision_ui([], [])
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/revision/2'}
+        rev_ui = branch_app.lookup_app(env)
+        rev_ui.parse_args(env)
+        self.assertIsInstance(rev_ui.get_values('', {}, []), dict)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n'), ('other-file', 'other\n')],
+                [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/revision/head:'}
+        revision_ui = branch_app.lookup_app(env)
+        revision_ui.parse_args(env)
+        values = revision_ui.get_values('', {}, {})
+
+        self.assertEqual(values['revid'], 'rev-2')
+        self.assertEqual(values['change'].comment, 'msg 2')
+        self.assertEqual(values['file_changes'].modified[0].filename, 'file')
+        self.assertEqual(values['merged_in'], None)
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_revision_ui(
+                [('file', 'content\n'), ('other-file', 'other\n')],
+                [('file', 'new content\n')])
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/revision/head:'}
+        revision_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(revision_ui, env)
+
 
 
 class TestAnnotateUI(BasicTests):
@@ -125,3 +137,70 @@
         self.assertEqual(2, len(annotated))
         self.assertEqual('2', annotated[1].change.revno)
         self.assertEqual('1', annotated[2].change.revno)
+
+
+class TestFileDiffUI(BasicTests):
+
+    def make_branch_app_for_filediff_ui(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-1-id', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('filename', 'f-id', 'file', 'content\n'))],
+            message="First commit.")
+        builder.build_snapshot('rev-2-id', None, [
+            ('modify', ('f-id', 'new content\n'))])
+        builder.finish_series()
+        branch = builder.get_branch()
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_filediff_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+filediff/rev-2-id/rev-1-id/f-id'}
+        filediff_ui = branch_app.lookup_app(env)
+        filediff_ui.parse_args(env)
+        values = filediff_ui.get_values('', {}, {})
+        chunks = values['chunks']
+        self.assertEqual('insert', chunks[0].diff[1].type)
+        self.assertEqual('new content', chunks[0].diff[1].line)
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_filediff_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+json/+filediff/rev-2-id/rev-1-id/f-id'}
+        filediff_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(filediff_ui, env)
+
+
+class TestRevLogUI(BasicTests):
+
+    def make_branch_app_for_revlog_ui(self):
+        builder = self.make_branch_builder('branch')
+        builder.start_series()
+        builder.build_snapshot('rev-id', None, [
+            ('add', ('', 'root-id', 'directory', '')),
+            ('add', ('filename', 'f-id', 'file', 'content\n'))],
+            message="First commit.")
+        builder.finish_series()
+        branch = builder.get_branch()
+        self.addCleanup(branch.lock_read().unlock)
+        return self.make_branch_app(branch)
+
+    def test_get_values_smoke(self):
+        branch_app = self.make_branch_app_for_revlog_ui()
+        env = {'SCRIPT_NAME': '/',
+               'PATH_INFO': '/+revlog/rev-id'}
+        revlog_ui = branch_app.lookup_app(env)
+        revlog_ui.parse_args(env)
+        values = revlog_ui.get_values('', {}, {})
+        self.assertEqual(values['file_changes'].added[1].filename, 'filename')
+        self.assertEqual(values['entry'].comment, "First commit.")
+
+    def test_json_render_smoke(self):
+        branch_app = self.make_branch_app_for_revlog_ui()
+        env = {'SCRIPT_NAME': '', 'PATH_INFO': '/+json/+revlog/rev-id'}
+        revlog_ui = branch_app.lookup_app(env)
+        self.assertOkJsonResponse(revlog_ui, env)
+

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2011-03-23 03:36:23 +0000
+++ loggerhead/tests/test_simple.py	2011-06-28 14:19:58 +0000
@@ -18,6 +18,8 @@
 import cgi
 import logging
 import re
+import simplejson
+from cStringIO import StringIO
 
 from bzrlib.tests import TestCaseWithTransport
 try:
@@ -47,6 +49,23 @@
         branch_app = BranchWSGIApp(self.tree.branch, '', **kw).app
         return TestApp(HTTPExceptionHandler(branch_app))
 
+    def assertOkJsonResponse(self, app, env):
+        start, content = consume_app(app, env)
+        self.assertEqual('200 OK', start[0])
+        self.assertEqual('application/json', dict(start[1])['Content-Type'])
+        self.assertEqual(None, start[2])
+        simplejson.loads(content)
+
+    def make_branch_app(self, branch):
+        branch_app = BranchWSGIApp(branch, friendly_name='friendly-name')
+        branch_app._environ = {
+            'wsgi.url_scheme':'',
+            'SERVER_NAME':'',
+            'SERVER_PORT':'80',
+            }
+        branch_app._url_base = ''
+        return branch_app
+
 
 class TestWithSimpleTree(BasicTests):
 
@@ -228,6 +247,18 @@
         self.assertEqualDiff('', res.body)
 
 
+def consume_app(app, env):
+    body = StringIO()
+    start = []
+    def start_response(status, headers, exc_info=None):
+        start.append((status, headers, exc_info))
+        return body.write
+    extra_content = list(app(env, start_response))
+    body.writelines(extra_content)
+    return start[0], body.getvalue()
+
+
+
 #class TestGlobalConfig(BasicTests):
 #    """
 #    Test that global config settings are respected

=== modified file 'loggerhead/util.py'
--- loggerhead/util.py	2011-03-23 05:21:34 +0000
+++ loggerhead/util.py	2011-06-27 14:17:38 +0000
@@ -663,3 +663,13 @@
             else:
                 raise
     return new_application
+
+
+def convert_to_json_ready(obj):
+    if isinstance(obj, Container):
+        d = obj.__dict__.copy()
+        del d['_properties']
+        return d
+    elif isinstance(obj, datetime.datetime):
+        return tuple(obj.utctimetuple())
+    raise TypeError(repr(obj) + " is not JSON serializable")

